#coding=utf-8
from datetime import datetime,date
from reportlab.graphics.barcode import code128
from reportlab.lib.units import inch
import StringIO
import csv
import json
import re
import xml.etree.ElementTree as ET

from django.core.cache import cache
from django.core.serializers.json import DjangoJSONEncoder
from django.db.models import Model
from django.http import HttpResponseNotAllowed, HttpResponse
from django.test.simple import DjangoTestSuiteRunner
from django.views.generic import View


class BarCode(code128.Code128, object):
    def __init__(self, value='', text = None, **kwd):
        self._setKeywords(**{
                             'humanReadable': 1,
                             'text': text,
                             'barWidth': inch * 0.015,
                             'barHeight': inch * 0.2,
        })
        super(BarCode, self).__init__(value = value, **kwd)
        
    def _humanText(self):
        if self.text != None:
            return self.text
        else:
            return super(BarCode, self)._humanText()


class NoDbTestRunner(DjangoTestSuiteRunner):
    """ A test runner to test without database creation """

    def setup_databases(self, **kwargs):
        """ Override the database creation defined in parent class """
        pass

    def teardown_databases(self, old_config, **kwargs):
        """ Override the database teardown defined in parent class """
        pass

class ProgressView(View):
    def get(self, request, *args, **kwargs):
        user = request.user
        if not user.is_authenticated():
            return HttpResponseNotAllowed([])
        
        progressname = kwargs.get('progressname', None) or request.GET.get('progressname', None)
        salt = request.GET.get('salt', 'user')
        if salt == 'user':
            key = user.id
        else:
            key = ''
        progresshandler = ProgressHandler(key)
        result = progresshandler.get(progressname)
        
        return HttpResponse(json_serialize(result))

class CJsonEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, date):
            return obj.strftime('%Y-%m-%d')
        else:
            return json.JSONEncoder.default(self, obj)

class ProgressHandler(object):
    salt = None #normally user id
    def __init__(self, salt = ''):
        self.salt = salt
    
    def initial(self, progressname, total = 0):
        if self.get(progressname) != None:
            #print 'One progress for the same salt is working:',progressname,self.get(progresskey)
            raise Exception('相同任务已经在执行！')
        else:
            self.set(progressname, 0, total)
        
    def getProgressKey(self, progressname):
        return str(self.salt) + '_' + str(progressname).replace(' ', '_')
    
    def get(self, progressname):
        progresskey = self.getProgressKey(progressname)
        result = cache.get(progresskey)
        #if result!= None and result['total'] == result['current']:
            #cache.delete(progresskey)
        #print 'getting progress:',progresskey,result
            
        return result
    
    def delete(self, progressname):
        progresskey = self.getProgressKey(progressname)
        cache.delete(progresskey)
        #print 'deleting progress:',progresskey,self.get(progressname)
    
    def set(self, progressname, current, total, timeout = 30):
        progresskey = self.getProgressKey(progressname)
        #print 'setting progress:',progresskey,current,total
        cache.set(progresskey, {'total': total, 'current': current}, timeout)

def get_element_by_tag(item,name1,name2=None):
    value = item.getElementsByTagName(name1)
    if len(value) ==0:
        value = ""
    elif name2 is None :
        try:
            value = value[0].childNodes[0].data
        except:
            value =""
    else:
        value = get_element_by_tag(value[0],name2)
    return str(value).strip()

def get_dict_from_object(instance, custom_dict = None, override_mode = True, combine = False):
    if hasattr(instance, '__dict__') and issubclass(instance.__dict__.__class__, type({})):
        item_dict = instance.__dict__.copy()
    elif issubclass(instance.__class__, type({})):
        item_dict = instance.copy()
    elif instance == None:
        return {}
    else:
        raise Exception('instance type %s not surported!'%str(type(instance)))
            
    if custom_dict != None:
        field_dict = {}
        if override_mode:
            fieldlist = item_dict.keys()
            for field in fieldlist:
                if field.startswith('_'):
                    continue
                field_dict[field] = field
        field_dict.update(custom_dict)
        item_dict = {}
        for k, a in field_dict.items():
            if combine:
                _loop_is = a.split('+')
            else:
                _loop_is = [a]
            _buf_x = []
            for _loop_i in _loop_is:
                attrs = _loop_i.split('.')
                tmp = instance
                for attr in  attrs:
                    tmp = getattr(tmp, attr.strip(), None) or (tmp.get(attr.strip(), None) if hasattr(tmp, 'get') else None)
                    if tmp == None:
                        tmp = ''
                        break
                if issubclass(tmp.__class__, Model):
                    tmp = tmp.pk
                _buf_x.append(str(tmp))
            item_dict[k] = ' '.join(_buf_x) if combine else tmp
    else:
        for key in item_dict.keys():
            if key.startswith('_'):
                item_dict.pop(key)
            
    return item_dict

def get_dicts_from_objects(instances, custom_dict = None, override_mode = True, combine = False):
    newdicts = [get_dict_from_object(instance, custom_dict = custom_dict, override_mode = override_mode, combine = combine) for instance in instances]
    
    return newdicts

def json_serialize(instance, custom_dict = None, override_mode = True, combine = False):
    if issubclass(instance.__class__, type({})) or not hasattr(instance, '__iter__'):
        return json.dumps(instance, ensure_ascii = False, cls = DjangoJSONEncoder)
    else:
        new_dicts = get_dicts_from_objects(instance, custom_dict = custom_dict, override_mode = override_mode, combine = combine)
                    
        itemstrs = [json.dumps(new_dict, ensure_ascii = False, cls = DjangoJSONEncoder) for new_dict in new_dicts]
        results = '[' + ','.join(itemstrs) + ']'
        
        return results

def get_csv_file_string(headers, recorddicts,**kwargs):
    csvstring = StringIO.StringIO()
    writter = csv.DictWriter(csvstring, headers,**kwargs)
    writter.writeheader()
    writter.writerows(recorddicts)
    
    return csvstring.getvalue()
    
class UnicodeCsvReader(object):
    def __init__(self, f, encoding="utf-8", **kwargs):
        self.csv_reader = csv.reader(f, **kwargs)
        self.encoding = encoding

    def __iter__(self):
        return self

    def next(self):
        # read and split the csv row into fields
        row = self.csv_reader.next() 
        # now decode
        return [unicode(cell, self.encoding) for cell in row]

    @property
    def line_num(self):
        return self.csv_reader.line_num

class UnicodeDictReader(csv.DictReader, object):
    def __init__(self, f, encoding="utf-8", fieldnames=None, **kwds):
        csv.DictReader.__init__(self, f, fieldnames=fieldnames, **kwds)
        self.reader = UnicodeCsvReader(f, encoding=encoding, **kwds)
        self.kwds = kwds
        self.encoding = encoding
        self.f = f
        self.subrows = []
        self.subrows_searched = False
        self.maxline_num = 0
        
        
    def issubrow(self, row):
        if not [v for k, v in row.items() if not self.issubfield(k) and v]:
            return True
        else:
            return False 
        
    def issubfield(self, field):
        if '/' in field:
            return True
        else:
            return False
        
    def search_subrows(self):
        while 1:
            try:
                row = super(UnicodeDictReader, self).next()
            except StopIteration:
                break
            self.maxline_num = self.line_num
            if self.issubrow(row):
                self.subrows.append(self.line_num)
        self.f.seek(0)
        self.f.readline()
        self.subrows_searched = True
        self.reader = UnicodeCsvReader(self.f, encoding=self.encoding, **self.kwds)
        
    def next(self):
        if not self.subrows_searched:
            self.search_subrows()
        row = super(UnicodeDictReader, self).next()
        if self.issubrow(row):
            row = super(UnicodeDictReader, self).next()
        for k, v in row.items():
            if self.issubfield(k):
                row[k] = [v]

        nextline_num = self.line_num + 1
        while nextline_num <= self.maxline_num:
            if nextline_num in self.subrows:
                nextrow = super(UnicodeDictReader, self).next()
                for k, v in row.items():
                    if self.issubfield(k):
                        row[k].append(nextrow[k])
            nextline_num = nextline_num + 1
                    
        return row

class object_dict(dict):
    """object view of dict, you can
    >>> a = object_dict()
    >>> a.fish = 'fish'
    >>> a['fish']
    'fish'
    >>> a['water'] = 'water'
    >>> a.water
    'water'
    >>> a.test = {'value': 1}
    >>> a.test2 = object_dict({'name': 'test2', 'value': 2})
    >>> a.test, a.test2.name, a.test2.value
    (1, 'test2', 2)
    """
    def __init__(self, initd=None):
        if initd is None:
            initd = {}
        dict.__init__(self, initd)

    def __getattr__(self, item):

        d = self.__getitem__(item)

        if isinstance(d, dict) and 'value' in d and len(d) == 1:
            return d['value']
        else:
            return d

    # if value is the only key in object, you can omit it
    def __setstate__(self, item):
        return False

    def __setattr__(self, item, value):
        self.__setitem__(item, value)

    def getvalue(self, item, value=None):
        return self.get(item, {}).get('value', value)

class xml2dict(object):

    def __init__(self):
        pass

    def _parse_node(self, node):
        node_tree = object_dict()
        # Save attrs and text, hope there will not be a child with same name
        if node.text:
            node_tree.value = node.text
        for (k, v) in node.attrib.items():
            k, v = self._namespace_split(k, object_dict({'value':v}))
            node_tree[k] = v
        #Save childrens
        for child in node.getchildren():     ###################DeprecationWarning: This method will be removed in future versions.  Use 'list(elem)' or iteration over elem instead.

            tag, tree = self._namespace_split(child.tag,
                                              self._parse_node(child))
            if tag not in node_tree:  # the first time, so store it in dict
                node_tree[tag] = tree
                continue
            old = node_tree[tag]
            if not isinstance(old, list):
                node_tree.pop(tag)
                node_tree[tag] = [old]  # multi times, so change old dict to a list
            node_tree[tag].append(tree)  # add the new one

        return node_tree

    def _namespace_split(self, tag, value):
        """
        Split the tag '{http://cs.sfsu.edu/csc867/myscheduler}patients'
        ns = http://cs.sfsu.edu/csc867/myscheduler
        name = patients
        """
        result = re.compile("\{(.*)\}(.*)").search(tag)
        if result:
            value.namespace, tag = result.groups()

        return (tag, value)

    def parse(self, xmlfile):
        """parse a xml file to a dict"""
        f = open(xmlfile, 'r')
        return self.fromstring(f.read())

    def fromstring(self, s):
        """parse a string"""
        t = ET.fromstring(s)
        root_tag, root_tree = self._namespace_split(t.tag, self._parse_node(t))
        return object_dict({root_tag: root_tree})

def mitbbs_check(username):
    '''
    检查mitbbs用户是否输入正确
    '''
    try:
    
        import urllib
        import urllib2
        
        data = {'userid' : str(username)}
        url = 'http://www.mitbbs.com/mitbbs_bbsqry.php'
    
        f = urllib2.urlopen(
                url     = url,
                data    = urllib.urlencode(data)
                )
        buf_str = f.read()
        
        if buf_str.__len__()>10000 and buf_str.__contains__('<script src="/js/jquery_min.js" type="text/javascript"></script>'):
            return True
        else:
            return False    
    except Exception,e:
        print e
        return False

UNIT_CONVERTION_DICT = {
    'oz':{
        'g': 28.3495231,
        'lb': 0.0625,
    },
    'g':{
        'oz': 0.035274,
        'lb': 0.0022046,
    },
    'lb':{
        'g': 453.59237,
        'oz': 16,
    }
}

def getUnitConvertion(unit, newunit = 'g'):
    if UNIT_CONVERTION_DICT.has_key(unit.lower())\
     and UNIT_CONVERTION_DICT[unit.lower()].has_key(newunit.lower()):
        return UNIT_CONVERTION_DICT[unit.lower()][newunit.lower()]
    else:
        return None

def unitConvert(number, unit, newunit = 'g'):
    if unit == newunit:
        return number
    else:
        unitconvertion = getUnitConvertion(unit, newunit)
        if unitconvertion == None:
            raise Exception('Unit convert failed!')
        else:
            return round(float(number) * unitconvertion, 2)

 
def get_client_ip(request):
    '''
    获取IP地址
    '''
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip

def dictExtend(olddict, newdict):
    olddict.update(newdict)
        
    return olddict

encoding_list = ['utf-8', 'iso-8859-1', 'gb2312']
def recode(stringvalue, toencoding = 'utf-8'):
    encoded = False
    for encoding in encoding_list:
        try:
            stringvalue = stringvalue.decode(encoding)
            encoded = True
            break
        except:
            continue
        
    if not encoded:
        raise Exception('%s is not surported in encoding list')
    else:
        return stringvalue.encode(toencoding)
    
def getNearestLoc(stringvalue, oldloc, encoding = 'utf-8'):
    i = oldloc
    while i > 0:
        try:
            stringvalue[0:i].decode(encoding)
            return i
        except:
            i = i - 1
            continue

    return 0

def getQueryWithPrefix(oldquery, prefix = ''):
    newquery = {}
    for k, v in oldquery.items():
        if prefix != '':
            newquery[prefix + '__' + k] = v
        else:
            newquery[k] = v
    
    return newquery 

